c----CAMx v5.20.1 1007014
c  
c     CHMDAT.COM contains all chemistry variables 
c                            
c     Copyright 1996 - 2010
c     ENVIRON International Corporation
c            
c     Modifications:  
c       4/4/00     Added aerosol deposition variables to /aerochm/
c       1/9/02     Aerosol size cut points and density now a function of
c                  species
c       8/20/02    Added minimum CWC to define presence of clouds
c      12/12/02    Expanded species list for Mechanism 4
c       1/10/03    Added array for deposition output species names
c      10/14/04    Modifications for mechanism 10
c      11/30/06    Created by taking Static variables out of chmstry.com
c      12/29/06 -bkoo-     Expanded species list for the updated SOA scheme
c      01/08/07 -bkoo-     Expanded species list for Mechanism 6 (CB05)
c                          Now ipigrxn is a scalar pointer to NO self-reaction
c      07/04/07 -bkoo-     Added pointer to hydrolysis of N2O5
c      04/28/08 -gyarwood- Added EBI solver
c 
c-----------------------------------------------------------------------
c     Parameters for some of the switches:
c
c     CDCMC  -- string for requesting the CMC chemistry solver
c     CDEBI  -- string for requesting the EBI chemistry solver
c     CDIEH  -- string for requesting the IEH chemistry solver
c     CDLSOD -- string for requesting the LSODE chemistry solver
c     IDCMC  -- code for using the CMC chemistry solver
c     IDEBI  -- code for using the EBI chemistry solver
c     IDIEH  -- code for using the IEH chemistry solver
c     IDLSOD -- code for using the LSODE chemistry solver
c
c-----------------------------------------------------------------------
c
      character*10 CDCMC
      character*10 CDEBI
      character*10 CDIEH
      character*10 CDLSOD
      integer      IDCMC
      integer      IDEBI
      integer      IDIEH
      integer      IDLSOD
c
      parameter( CDCMC  = "CMC       " )
      parameter( CDEBI  = "EBI       " )
      parameter( CDIEH  = "IEH       " )
      parameter( CDLSOD = "LSODE     " )
      parameter( IDCMC  = 1 )
      parameter( IDEBI  = 2 )
      parameter( IDIEH  = 3 )
      parameter( IDLSOD = 4 )
c 
c-----------------------------------------------------------------------
c    Variables for the number of species in input files:
c
c    ngas   --  number of gas species being modeled
c    naero  --  number of aersol species being modeled
c    nspec  --  total number of modeled species
c    nrad   --  number of radical species being modeled
c    nreact --  number of chemical reactions
c    nspfst --  number of "fast" species -- handled by the fast solver
c    iessrad--  number of radicals in steady state for IEH solver
c    idmech --  the code which determines which chemical mechanism is used
c    idsolv --  the code which determines which chemstry solver to use
c    navspc --  number of species to write to output average file
c    ndepspc--  number of species to write to deposition file
c    nicspc --  number of species in the initial conditions file
c    nbcspc --  number of species in the boundary conditions file
c    nptspc --  number of species in the point source emissions file
c
c-----------------------------------------------------------------------
c
       integer   ngas
       integer   naero
       integer   nspec
       integer   nrad
       integer   nreact
       integer   nspfst
       integer   iessrad
       integer   idmech
       integer   idsolv
       integer   navspc
       integer   ndepspc
       integer   nicspc
       integer   nbcspc
       integer   nptspc
c
      common /chm1/ ngas, naero, nspec, nrad, nreact, nspfst, iessrad,
     &                         idmech, idsolv, navspc, ndepspc, nicspc,
     &                                                    nbcspc, nptspc
c
c-----------------------------------------------------------------------
c
c    aeropt --  keyword for aerosol scheme ('CF' or 'CMU')
c
c-----------------------------------------------------------------------
c
      character*10 aeropt
c
      common /chm2/ aeropt
c
c-----------------------------------------------------------------------
c     Variables for storing chemical reaction data:
c
c     bdlrad -- lower bound value for each radical species (ppm)
c
c-----------------------------------------------------------------------
c
      real bdlrad
c
      common /chmrate/ bdlrad
c
c-----------------------------------------------------------------------
c     Variables for photolysis data:
c
c     nphot1   -- number of primary photolysis reactions
c     nphot2   -- number of secondary (scaled) photolysis reactions
c
c-----------------------------------------------------------------------
c
      integer nphot1
      integer nphot2
c
      common /photmap/ nphot1, nphot2
c 
c-----------------------------------------------------------------------
c     Variables for vertical profiles of HCl and CL2 for Hg chemistry
c
c     NHTCL   --  number of heights for concentrations 
c     htcl    --  heights (km) for the top of each concentration range
c     hclprof --  profile of HCl (ppm) with height
c     cl2day  --  profile of Cl2 (ppm) with height during day over ocean
c     cl2nite --  profile of Cl2 (ppm) with height at night over ocean
c
c-----------------------------------------------------------------------
c
      integer NHTCL
c
      parameter ( NHTCL = 6 )
c
      real htcl(NHTCL)
      real hclprof(NHTCL)
      real cl2day(NHTCL)
      real cl2nite(NHTCL)
c
      common /clprofs/ htcl, hclprof, cl2day, cl2nite
c 
c-----------------------------------------------------------------------
c     Parameters for mapping input species to internal model order:
c
c      NSPNAM  -- number of recognizable state species
c      NRADNM  -- number of recognizable radical species
c
      integer   NSPNAM
      integer   NRADNM
c
      parameter ( NSPNAM  =   305 )
      parameter ( NRADNM  =    26 )
c 
c-----------------------------------------------------------------------
c     Variables for mapping input species to internal model order:
c
c     krad     -- mapping of radical species to specific mechanism order
c     kmap     -- mapping of species on chemistry parameters file to
c                 internal order
c
c-----------------------------------------------------------------------
c
      integer krad(NRADNM)
      integer kmap(NSPNAM)
c
      common /kname/ krad, kmap
c
      integer        kno,    kno2,     ko3
      integer       kpan,   kpanx,   kcres
      integer      kpan2,   kmpan,   kpbzn
      integer      knphe,   krno3,   kdcb2
      integer      kdcb3,   khno4,   kaacd
      integer      kacet,   kald2,   kaldx
      integer      kalk1,   kalk2,   kalk3
      integer      kalk4,   kalk5,   karo1
      integer      karo2,   kbacl,   kbald
      integer      kbcl1,   kbcl2,   kbuta
      integer      kccho,   kccrs,    kcg1
      integer       kcg2,    kcg3,    kcg4
      integer       kcg5,    kcg6,    kcg7
      integer       kcl2,     kco,   kco2h
      integer      kco3h,   kcooh,   kcprm
      integer      kdcb1,    keth,   ketha
      integer      kethe,   ketoh,   kfacd
      integer      kfcrs,   kfmcl,   kform
      integer      kfprm,    kgly,   kh2o2
      integer      khc2h,   khcho,    khcl
      integer       khg0,    khg2,    khgp
      integer      khno3,   kho2h,   khocl
      integer      khono,   kicl1,   kicl2
      integer      kiole,   kisop,    kisp
      integer      kispd,   kmbut,    kmek
      integer      kmeoh,   kmepx,   kmeth
      integer      kmgly,   kmtbe,    kmvk
      integer      kn2o5,     kna,    knh3
      integer       kno3,    kntr,    kole
      integer      kole1,   kole2,   kopen
      integer      kpacd,    kpar,    kpcl
      integer       kpec,   kph2o,   kphen
      integer       kpna,   kpnh4,   kpno3
      integer       kpoa,   kprod,   kpso4
      integer      krc2h,   krc3h,   krcho
      integer      krooh,    kso2,   ksoa1
      integer      ksoa2,   ksoa3,   ksoa4
      integer      ksoa5,   ksoa6,   ksoa7
      integer      ksopa,   ksopb,    ksqt
      integer      ksulf,   kterp,    ktol
      integer      ktola,    ktrp,     kxn
      integer       kxyl,   kxyla, ksoa1_1
      integer    ksoa1_2, ksoa1_3, ksoa1_4
      integer    ksoa1_5, ksoa1_6, ksoa1_7
      integer    ksoa1_8, ksoa1_9,ksoa1_10
      integer    ksoa2_1, ksoa2_2, ksoa2_3
      integer    ksoa2_4, ksoa2_5, ksoa2_6
      integer    ksoa2_7, ksoa2_8, ksoa2_9
      integer   ksoa2_10, ksoa3_1, ksoa3_2
      integer    ksoa3_3, ksoa3_4, ksoa3_5
      integer    ksoa3_6, ksoa3_7, ksoa3_8
      integer    ksoa3_9,ksoa3_10, ksoa4_1
      integer    ksoa4_2, ksoa4_3, ksoa4_4
      integer    ksoa4_5, ksoa4_6, ksoa4_7
      integer    ksoa4_8, ksoa4_9,ksoa4_10
      integer    ksoa5_1, ksoa5_2, ksoa5_3
      integer    ksoa5_4, ksoa5_5, ksoa5_6
      integer    ksoa5_7, ksoa5_8, ksoa5_9
      integer   ksoa5_10, ksoa6_1, ksoa6_2
      integer    ksoa6_3, ksoa6_4, ksoa6_5
      integer    ksoa6_6, ksoa6_7, ksoa6_8
      integer    ksoa6_9,ksoa6_10, ksoa7_1
      integer    ksoa7_2, ksoa7_3, ksoa7_4
      integer    ksoa7_5, ksoa7_6, ksoa7_7
      integer    ksoa7_8, ksoa7_9,ksoa7_10
      integer    ksopa_1, ksopa_2, ksopa_3
      integer    ksopa_4, ksopa_5, ksopa_6
      integer    ksopa_7, ksopa_8, ksopa_9
      integer   ksopa_10, ksopb_1, ksopb_2
      integer    ksopb_3, ksopb_4, ksopb_5
      integer    ksopb_6, ksopb_7, ksopb_8
      integer    ksopb_9,ksopb_10,  kpoa_1
      integer     kpoa_2,  kpoa_3,  kpoa_4
      integer     kpoa_5,  kpoa_6,  kpoa_7
      integer     kpoa_8,  kpoa_9, kpoa_10
      integer     kpec_1,  kpec_2,  kpec_3
      integer     kpec_4,  kpec_5,  kpec_6
      integer     kpec_7,  kpec_8,  kpec_9
      integer    kpec_10, kcrst_1, kcrst_2
      integer    kcrst_3, kcrst_4, kcrst_5
      integer    kcrst_6, kcrst_7, kcrst_8
      integer    kcrst_9,kcrst_10, kph2o_1
      integer    kph2o_2, kph2o_3, kph2o_4
      integer    kph2o_5, kph2o_6, kph2o_7
      integer    kph2o_8, kph2o_9,kph2o_10
      integer     kpcl_1,  kpcl_2,  kpcl_3
      integer     kpcl_4,  kpcl_5,  kpcl_6
      integer     kpcl_7,  kpcl_8,  kpcl_9
      integer    kpcl_10,   kna_1,   kna_2
      integer      kna_3,   kna_4,   kna_5
      integer      kna_6,   kna_7,   kna_8
      integer      kna_9,  kna_10, kpnh4_1
      integer    kpnh4_2, kpnh4_3, kpnh4_4
      integer    kpnh4_5, kpnh4_6, kpnh4_7
      integer    kpnh4_8, kpnh4_9,kpnh4_10
      integer    kpno3_1, kpno3_2, kpno3_3
      integer    kpno3_4, kpno3_5, kpno3_6
      integer    kpno3_7, kpno3_8, kpno3_9
      integer   kpno3_10, kpso4_1, kpso4_2
      integer    kpso4_3, kpso4_4, kpso4_5
      integer    kpso4_6, kpso4_7, kpso4_8
      integer    kpso4_9,kpso4_10
c
      equivalence
     &   (kmap(  1),     kno),(kmap(  2),    kno2),(kmap(  3),     ko3),
     &   (kmap(  4),    kpan),(kmap(  5),   kpanx),(kmap(  6),   kcres),
     &   (kmap(  7),   kpan2),(kmap(  8),   kmpan),(kmap(  9),   kpbzn),
     &   (kmap( 10),   knphe),(kmap( 11),   krno3),(kmap( 12),   kdcb2),
     &   (kmap( 13),   kdcb3),(kmap( 14),   khno4),(kmap( 15),   kaacd),
     &   (kmap( 16),   kacet),(kmap( 17),   kald2),(kmap( 18),   kaldx),
     &   (kmap( 19),   kalk1),(kmap( 20),   kalk2),(kmap( 21),   kalk3),
     &   (kmap( 22),   kalk4),(kmap( 23),   kalk5),(kmap( 24),   karo1),
     &   (kmap( 25),   karo2),(kmap( 26),   kbacl),(kmap( 27),   kbald),
     &   (kmap( 28),   kbcl1),(kmap( 29),   kbcl2),(kmap( 30),   kbuta),
     &   (kmap( 31),   kccho),(kmap( 32),   kccrs),(kmap( 33),    kcg1),
     &   (kmap( 34),    kcg2),(kmap( 35),    kcg3),(kmap( 36),    kcg4),
     &   (kmap( 37),    kcg5),(kmap( 38),    kcg6),(kmap( 39),    kcg7),
     &   (kmap( 40),    kcl2),(kmap( 41),     kco),(kmap( 42),   kco2h),
     &   (kmap( 43),   kco3h),(kmap( 44),   kcooh),(kmap( 45),   kcprm),
     &   (kmap( 46),   kdcb1),(kmap( 47),    keth),(kmap( 48),   ketha),
     &   (kmap( 49),   kethe),(kmap( 50),   ketoh),(kmap( 51),   kfacd),
     &   (kmap( 52),   kfcrs),(kmap( 53),   kfmcl),(kmap( 54),   kform),
     &   (kmap( 55),   kfprm),(kmap( 56),    kgly),(kmap( 57),   kh2o2),
     &   (kmap( 58),   khc2h),(kmap( 59),   khcho),(kmap( 60),    khcl),
     &   (kmap( 61),    khg0),(kmap( 62),    khg2),(kmap( 63),    khgp),
     &   (kmap( 64),   khno3),(kmap( 65),   kho2h),(kmap( 66),   khocl),
     &   (kmap( 67),   khono),(kmap( 68),   kicl1),(kmap( 69),   kicl2),
     &   (kmap( 70),   kiole),(kmap( 71),   kisop),(kmap( 72),    kisp),
     &   (kmap( 73),   kispd),(kmap( 74),   kmbut),(kmap( 75),    kmek),
     &   (kmap( 76),   kmeoh),(kmap( 77),   kmepx),(kmap( 78),   kmeth),
     &   (kmap( 79),   kmgly),(kmap( 80),   kmtbe),(kmap( 81),    kmvk),
     &   (kmap( 82),   kn2o5),(kmap( 83),     kna),(kmap( 84),    knh3),
     &   (kmap( 85),    kno3),(kmap( 86),    kntr),(kmap( 87),    kole),
     &   (kmap( 88),   kole1),(kmap( 89),   kole2),(kmap( 90),   kopen),
     &   (kmap( 91),   kpacd),(kmap( 92),    kpar),(kmap( 93),    kpcl),
     &   (kmap( 94),    kpec),(kmap( 95),   kph2o),(kmap( 96),   kphen),
     &   (kmap( 97),    kpna),(kmap( 98),   kpnh4),(kmap( 99),   kpno3),
     &   (kmap(100),    kpoa),(kmap(101),   kprod),(kmap(102),   kpso4),
     &   (kmap(103),   krc2h),(kmap(104),   krc3h),(kmap(105),   krcho),
     &   (kmap(106),   krooh),(kmap(107),    kso2),(kmap(108),   ksoa1),
     &   (kmap(109),   ksoa2),(kmap(110),   ksoa3),(kmap(111),   ksoa4),
     &   (kmap(112),   ksoa5),(kmap(113),   ksoa6),(kmap(114),   ksoa7),
     &   (kmap(115),   ksopa),(kmap(116),   ksopb),(kmap(117),    ksqt),
     &   (kmap(118),   ksulf),(kmap(119),   kterp),(kmap(120),    ktol),
     &   (kmap(121),   ktola),(kmap(122),    ktrp),(kmap(123),     kxn),
     &   (kmap(124),    kxyl),(kmap(125),   kxyla),(kmap(126), ksoa1_1),
     &   (kmap(127), ksoa1_2),(kmap(128), ksoa1_3),(kmap(129), ksoa1_4),
     &   (kmap(130), ksoa1_5),(kmap(131), ksoa1_6),(kmap(132), ksoa1_7),
     &   (kmap(133), ksoa1_8),(kmap(134), ksoa1_9),(kmap(135),ksoa1_10),
     &   (kmap(136), ksoa2_1),(kmap(137), ksoa2_2),(kmap(138), ksoa2_3),
     &   (kmap(139), ksoa2_4),(kmap(140), ksoa2_5),(kmap(141), ksoa2_6),
     &   (kmap(142), ksoa2_7),(kmap(143), ksoa2_8),(kmap(144), ksoa2_9),
     &   (kmap(145),ksoa2_10),(kmap(146), ksoa3_1),(kmap(147), ksoa3_2),
     &   (kmap(148), ksoa3_3),(kmap(149), ksoa3_4),(kmap(150), ksoa3_5),
     &   (kmap(151), ksoa3_6),(kmap(152), ksoa3_7),(kmap(153), ksoa3_8),
     &   (kmap(154), ksoa3_9),(kmap(155),ksoa3_10),(kmap(156), ksoa4_1),
     &   (kmap(157), ksoa4_2),(kmap(158), ksoa4_3),(kmap(159), ksoa4_4),
     &   (kmap(160), ksoa4_5),(kmap(161), ksoa4_6),(kmap(162), ksoa4_7),
     &   (kmap(163), ksoa4_8),(kmap(164), ksoa4_9),(kmap(165),ksoa4_10),
     &   (kmap(166), ksoa5_1),(kmap(167), ksoa5_2),(kmap(168), ksoa5_3),
     &   (kmap(169), ksoa5_4),(kmap(170), ksoa5_5),(kmap(171), ksoa5_6),
     &   (kmap(172), ksoa5_7),(kmap(173), ksoa5_8),(kmap(174), ksoa5_9),
     &   (kmap(175),ksoa5_10),(kmap(176), ksoa6_1),(kmap(177), ksoa6_2),
     &   (kmap(178), ksoa6_3),(kmap(179), ksoa6_4),(kmap(180), ksoa6_5),
     &   (kmap(181), ksoa6_6),(kmap(182), ksoa6_7),(kmap(183), ksoa6_8),
     &   (kmap(184), ksoa6_9),(kmap(185),ksoa6_10),(kmap(186), ksoa7_1),
     &   (kmap(187), ksoa7_2),(kmap(188), ksoa7_3),(kmap(189), ksoa7_4),
     &   (kmap(190), ksoa7_5),(kmap(191), ksoa7_6),(kmap(192), ksoa7_7),
     &   (kmap(193), ksoa7_8),(kmap(194), ksoa7_9),(kmap(195),ksoa7_10),
     &   (kmap(196), ksopa_1),(kmap(197), ksopa_2),(kmap(198), ksopa_3),
     &   (kmap(199), ksopa_4),(kmap(200), ksopa_5),(kmap(201), ksopa_6),
     &   (kmap(202), ksopa_7),(kmap(203), ksopa_8),(kmap(204), ksopa_9),
     &   (kmap(205),ksopa_10),(kmap(206), ksopb_1),(kmap(207), ksopb_2),
     &   (kmap(208), ksopb_3),(kmap(209), ksopb_4),(kmap(210), ksopb_5),
     &   (kmap(211), ksopb_6),(kmap(212), ksopb_7),(kmap(213), ksopb_8),
     &   (kmap(214), ksopb_9),(kmap(215),ksopb_10),(kmap(216),  kpoa_1),
     &   (kmap(217),  kpoa_2),(kmap(218),  kpoa_3),(kmap(219),  kpoa_4),
     &   (kmap(220),  kpoa_5),(kmap(221),  kpoa_6),(kmap(222),  kpoa_7),
     &   (kmap(223),  kpoa_8),(kmap(224),  kpoa_9),(kmap(225), kpoa_10),
     &   (kmap(226),  kpec_1),(kmap(227),  kpec_2),(kmap(228),  kpec_3),
     &   (kmap(229),  kpec_4),(kmap(230),  kpec_5),(kmap(231),  kpec_6),
     &   (kmap(232),  kpec_7),(kmap(233),  kpec_8),(kmap(234),  kpec_9),
     &   (kmap(235), kpec_10),(kmap(236), kcrst_1),(kmap(237), kcrst_2),
     &   (kmap(238), kcrst_3),(kmap(239), kcrst_4),(kmap(240), kcrst_5),
     &   (kmap(241), kcrst_6),(kmap(242), kcrst_7),(kmap(243), kcrst_8),
     &   (kmap(244), kcrst_9),(kmap(245),kcrst_10),(kmap(246), kph2o_1),
     &   (kmap(247), kph2o_2),(kmap(248), kph2o_3),(kmap(249), kph2o_4),
     &   (kmap(250), kph2o_5),(kmap(251), kph2o_6),(kmap(252), kph2o_7),
     &   (kmap(253), kph2o_8),(kmap(254), kph2o_9),(kmap(255),kph2o_10),
     &   (kmap(256),  kpcl_1),(kmap(257),  kpcl_2),(kmap(258),  kpcl_3),
     &   (kmap(259),  kpcl_4),(kmap(260),  kpcl_5),(kmap(261),  kpcl_6),
     &   (kmap(262),  kpcl_7),(kmap(263),  kpcl_8),(kmap(264),  kpcl_9),
     &   (kmap(265), kpcl_10),(kmap(266),   kna_1),(kmap(267),   kna_2),
     &   (kmap(268),   kna_3),(kmap(269),   kna_4),(kmap(270),   kna_5),
     &   (kmap(271),   kna_6),(kmap(272),   kna_7),(kmap(273),   kna_8),
     &   (kmap(274),   kna_9),(kmap(275),  kna_10),(kmap(276), kpnh4_1),
     &   (kmap(277), kpnh4_2),(kmap(278), kpnh4_3),(kmap(279), kpnh4_4),
     &   (kmap(280), kpnh4_5),(kmap(281), kpnh4_6),(kmap(282), kpnh4_7),
     &   (kmap(283), kpnh4_8),(kmap(284), kpnh4_9),(kmap(285),kpnh4_10),
     &   (kmap(286), kpno3_1),(kmap(287), kpno3_2),(kmap(288), kpno3_3),
     &   (kmap(289), kpno3_4),(kmap(290), kpno3_5),(kmap(291), kpno3_6),
     &   (kmap(292), kpno3_7),(kmap(293), kpno3_8),(kmap(294), kpno3_9),
     &   (kmap(295),kpno3_10),(kmap(296), kpso4_1),(kmap(297), kpso4_2),
     &   (kmap(298), kpso4_3),(kmap(299), kpso4_4),(kmap(300), kpso4_5),
     &   (kmap(301), kpso4_6),(kmap(302), kpso4_7),(kmap(303), kpso4_8),
     &   (kmap(304), kpso4_9),(kmap(305),kpso4_10)
c
      integer       ko1d,      ko,    kclo
      integer        kcl,     koh,    kho2
      integer      kc2o3,    kxo2,   kxo2n
      integer      kcxo3,   kmeo2,    kto2
      integer       kror,    kcro,   kro2r
      integer      kr2o2,   kro2n,   kcco3
      integer      krco3,   kmco3,   kbzco
      integer      kcxo2,   khco3,   ktbuo
      integer       kbzo,   kbzno
c
      equivalence
     &   (krad(  1),    ko1d),(krad(  2),      ko),(krad(  3),    kclo),
     &   (krad(  4),     kcl),(krad(  5),     koh),(krad(  6),    kho2),
     &   (krad(  7),   kc2o3),(krad(  8),    kxo2),(krad(  9),   kxo2n),
     &   (krad( 10),   kcxo3),(krad( 11),   kmeo2),(krad( 12),    kto2),
     &   (krad( 13),    kror),(krad( 14),    kcro),(krad( 15),   kro2r),
     &   (krad( 16),   kr2o2),(krad( 17),   kro2n),(krad( 18),   kcco3),
     &   (krad( 19),   krco3),(krad( 20),   kmco3),(krad( 21),   kbzco),
     &   (krad( 22),   kcxo2),(krad( 23),   khco3),(krad( 24),   ktbuo),
     &   (krad( 25),    kbzo),(krad( 26),   kbzno)
c
c-----------------------------------------------------------------------
c     Variables to define parameters for each chemical species:
c
c     henso20  -- Henry's Law constant at STP for SO2 (molar/atm)
c     tfactso2 -- Temperature dependence of SO2 Henry's Law constant (1/K)
c     nbin     -- Number of aerosol size bins
c     cwmin    -- Minimum cloud water threshold (g/m3)
c     tamin    -- Cloud water freezing threshold (K)
c
c-----------------------------------------------------------------------
c
      real henso20
      real tfactso2
      real cwmin
      real tamin
c
      common /depchm/ henso20, tfactso2, cwmin, tamin
c
      integer nbin
c
      common /aerochm/ nbin
c
c-----------------------------------------------------------------------
c     Pointer to hydrolysis of N2O5
c                    N2O5 + H2O -> 2 HNO3
c
      integer ihydrxn
      common /hydrxn/ ihydrxn
c
c-----------------------------------------------------------------------
c     Pointer used to lookup pig chemistry rate constant
c
c     ipigrxn  -- pointer to NO self-reaction
c                    NO + NO -> 2 NO2
c
      integer ipigrxn
c
      common /pigrxn/ ipigrxn
c
c----------------------------------------------------------------------
c    Variables for controlling calls to aerosol routines
c
c     dtaero     -- user input time interval (min) between calls to aerosol 
c                   routines
c     dt_aero    -- adjusted (for I/O frequency) time interval (min) between
c                   calls to aerosol routines
c
      real dtaero
      real dt_aero
c
      common /aero_t/ dtaero, dt_aero
